import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For integers between Number.MIN_SAFE_INTEGER (included) and Number.MAX_SAFE_INTEGER (included)
 * @remarks Since 1.11.0
 * @public
 */
export declare function maxSafeInteger(): Arbitrary<number>;
