import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For single base64 characters - A-Z, a-z, 0-9, + or /
 * @deprecated Prefer using `fc.constantFrom(...'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789+/')`
 * @remarks Since 0.0.1
 * @public
 */
export declare function base64(): Arbitrary<string>;
