import { buildUriQueryOrFragmentArbitrary } from './_internals/builders/UriQueryOrFragmentArbitraryBuilder.js';
export function webQueryParameters(constraints = {}) {
    return buildUriQueryOrFragmentArbitrary(constraints.size);
}
