import { anyArbitraryBuilder } from './_internals/builders/AnyArbitraryBuilder.js';
import { toQualifiedObjectConstraints } from './_internals/helpers/QualifiedObjectConstraints.js';
function anything(constraints) {
    return anyArbitraryBuilder(toQualifiedObjectConstraints(constraints));
}
export { anything };
