/**
 * Random generators automatically recognized by the framework
 * without having to pass a builder function
 * @remarks Since 2.2.0
 * @public
 */
export type RandomType = 'mersenne' | 'congruential' | 'congruential32' | 'xorshift128plus' | 'xoroshiro128plus';
