import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For comparison boolean functions
 *
 * A comparison boolean function returns:
 * - `true` whenever `a < b`
 * - `false` otherwise (ie. `a = b` or `a > b`)
 *
 * @remarks Since 1.6.0
 * @public
 */
export declare function compareBooleanFunc<T>(): Arbitrary<(a: T, b: T) => boolean>;
