import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For valid IP v6
 *
 * Following {@link https://tools.ietf.org/html/rfc3986#section-3.2.2 | RFC 3986}
 *
 * @remarks Since 1.14.0
 * @public
 */
export declare function ipV6(): Arbitrary<string>;
