/**
 * CRC-32 based hash function
 *
 * Used internally by fast-check in {@link func}, {@link compareFunc} or even {@link compareBooleanFunc}.
 *
 * @param repr - String value to be hashed
 *
 * @remarks Since 2.1.0
 * @public
 */
export declare function hash(repr: string): number;
