"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.char = char;
const CharacterArbitraryBuilder_1 = require("./_internals/builders/CharacterArbitraryBuilder");
function identity(v) {
    return v;
}
function char() {
    return (0, CharacterArbitraryBuilder_1.buildCharacterArbitrary)(0x20, 0x7e, identity, identity);
}
