"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.configureGlobal = configureGlobal;
exports.readConfigureGlobal = readConfigureGlobal;
exports.resetConfigureGlobal = resetConfigureGlobal;
let globalParameters = {};
function configureGlobal(parameters) {
    globalParameters = parameters;
}
function readConfigureGlobal() {
    return globalParameters;
}
function resetConfigureGlobal() {
    globalParameters = {};
}
