"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pre = pre;
const PreconditionFailure_1 = require("./PreconditionFailure");
function pre(expectTruthy) {
    if (!expectTruthy) {
        throw new PreconditionFailure_1.PreconditionFailure();
    }
}
