import type { ICommand } from './ICommand.js';
/**
 * Interface that should be implemented in order to define
 * a synchronous command
 *
 * @remarks Since 1.5.0
 * @public
 */
export interface Command<Model extends object, Real> extends ICommand<Model, Real, void> {
}
