import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For `value`
 * @param value - The value to produce
 * @remarks Since 0.0.1
 * @public
 */
export declare function constant<T>(value: T): Arbitrary<T>;
