import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For ulid
 *
 * According to {@link https://github.com/ulid/spec | ulid spec}
 *
 * No mixed case, only upper case digits (0-9A-Z except for: I,L,O,U)
 *
 * @remarks Since 3.11.0
 * @public
 */
export declare function ulid(): Arbitrary<string>;
