import type { ICommand } from './ICommand.js';
/**
 * Interface that should be implemented in order to define
 * an asynchronous command
 *
 * @remarks Since 1.5.0
 * @public
 */
export interface AsyncCommand<Model extends object, Real, CheckAsync extends boolean = false> extends ICommand<Model, Real, Promise<void>, CheckAsync> {
}
