import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For boolean values - `true` or `false`
 * @remarks Since 0.0.6
 * @public
 */
declare function boolean(): Arbitrary<boolean>;
export { boolean };
