import { RandomGenerator } from '../generator/RandomGenerator.js';
import { ArrayInt } from './internals/ArrayInt.js';
export declare function unsafeUniformArrayIntDistribution(from: ArrayInt, to: ArrayInt, rng: RandomGenerator): ArrayInt;
