"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.gen = gen;
const GeneratorArbitrary_1 = require("./_internals/GeneratorArbitrary");
function gen() {
    return new GeneratorArbitrary_1.GeneratorArbitrary();
}
