"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unicodeJson = unicodeJson;
const unicodeJsonValue_1 = require("./unicodeJsonValue");
function unicodeJson(constraints = {}) {
    const arb = (0, unicodeJsonValue_1.unicodeJsonValue)(constraints);
    return arb.map(JSON.stringify);
}
