import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
/**
 * For single hexadecimal characters - 0-9 or a-f
 * @deprecated Prefer using `fc.constantFrom(...'0123456789abcdef')`
 * @remarks Since 0.0.1
 * @public
 */
export declare function hexa(): Arbitrary<string>;
