import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
import type { IntArrayConstraints } from './_internals/builders/TypedIntArrayArbitraryBuilder.js';
/**
 * For Int8Array
 * @remarks Since 2.9.0
 * @public
 */
export declare function int8Array(constraints?: IntArrayConstraints): Arbitrary<Int8Array>;
export type { IntArrayConstraints };
