import type { Arbitrary } from '../check/arbitrary/definition/Arbitrary.js';
import type { BigIntArrayConstraints } from './_internals/builders/TypedIntArrayArbitraryBuilder.js';
/**
 * For BigUint64Array
 * @remarks Since 3.0.0
 * @public
 */
export declare function bigUint64Array(constraints?: BigIntArrayConstraints): Arbitrary<BigUint64Array>;
export type { BigIntArrayConstraints };
